# Copyright 2023 Canonical Ltd.
# Licensed under the Apache V2, see LICENCE file for details.

import logging

from . import model

log = logging.getLogger(__name__)


class Annotation(model.ModelEntity):
    pass
